package smash.appointment.parse;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class AppointmentEntry {
	private String								day;
	private String								time;
	private String								duration;
	private Subject								subject;
	private Set<AppointmentType>	types	= new HashSet<>();
	private String								source;

	/**
	 * @return the time
	 * @see #time
	 */
	public String getTime() {
		if (time == null || time.trim().isEmpty()) {
			return "09:00";
		} else {
			return time;
		}
	}

	/**
	 * @param time
	 *          the time to set
	 * @see #time
	 */
	public void setTime(String time) {
		this.time = time;
	}

	/**
	 * @return the subject
	 * @see #subject
	 */
	public Subject getSubject() {
		return subject;
	}

	/**
	 * @param subject
	 *          the subject to set
	 * @see #subject
	 */
	public void setSubject(Subject subject) {
		this.subject = subject;
	}

	/**
	 * @return the type
	 * @see #type
	 */
	public Set<AppointmentType> getTypes() {
		return types;
	}

	/**
	 * @param type
	 *          the type to set
	 * @see #type
	 */
	public void addType(AppointmentType type) {
		this.types.add(type);
	}

	/**
	 * @return the day
	 * @see #day
	 */
	public String getDay() {
		return day;
	}

	/**
	 * @param day
	 *          the day to set
	 * @see #day
	 */
	public void setDay(String day) {
		this.day = day;
	}

	/**
	 * @return the source
	 * @see #source
	 */
	public String getSource() {
		return source;
	}

	/**
	 * @param source
	 *          the source to set
	 * @see #source
	 */
	public void setSource(String source) {
		this.source = source;
	}

	@Override
	public String toString() {
		return day + " " + time + " " + subject + " " + types + "\t\t[source: " + source + "]";
	}

	/**
	 * @return the duration
	 * @see #duration
	 */
	public String getDuration() {
		if (duration == null || duration.trim().isEmpty()) {
			int count = 0;
			for (AppointmentType type : types) {
				count += type.getTime();
			}
			return count + "";
		}
		return duration;
	}

	/**
	 * @param duration
	 *          the duration to set
	 * @see #duration
	 */
	public void setDuration(String duration) {
		this.duration = duration;
	}

	public void addTypes(AppointmentType[] typesToAdd) {
		for (AppointmentType appointmentType : typesToAdd) {
			addType(appointmentType);
		}

	}

	public void addTypes(Collection<AppointmentType> typesToAdd) {
		for (AppointmentType appointmentType : typesToAdd) {
			addType(appointmentType);
		}

	}
}
