package smash.appointment.parse;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.Test;

public class LihControlMappingParserTest extends TestBase {
	Logger						logger		= Logger.getLogger(LihControlMappingParserTest.class);

	LihControlMappingParser processor	= new LihControlMappingParser();


	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp()  {
		super.setUp();
	}

	@After
	public void tearDown() throws Exception {
	}

	private static final SimpleDateFormat	DATE_FORMATTER = new SimpleDateFormat("yyyy-MM-dd");

	@Test
	public void test() throws Exception {
		List<Subject> entries = processor.processExcel("testFiles/lihControlMappingExample.xlsx");
		assertTrue(entries.size() > 0);

		Subject subject = entries.get(0);
		assertEquals("L-001", subject.getScreeningNumber());
		assertEquals("Piotrek", subject.getName());
		assertEquals("Gawron", subject.getSurname());
		assertEquals("", subject.getAddress());
		assertEquals("", subject.getZipCode());
		assertEquals("", subject.getCity());
		assertEquals("", subject.getCountry());
		assertEquals("343", subject.getPhone1());
		assertEquals("", subject.getPhone2());
		assertEquals("", subject.getPhone3());
		assertEquals("", subject.getMail());
		assertEquals("", subject.getDiagnosisYear());
		assertEquals("", subject.getDiagnosis());
		assertEquals("", subject.getReferal());
		assertEquals("2015-08-03", subject.getAddDate());
		assertEquals("ND3333", subject.getNdNumber());
		assertEquals("", subject.getBirthDate());
		
		for (Subject s:entries) {
			DATE_FORMATTER.parse(s.getAddDate());
		}
	}


	@Test
	public void testFixDate() throws Exception {
		DATE_FORMATTER.parse(processor.fixDate("2016-cot-31"));
		Date d = DATE_FORMATTER.parse(processor.fixDate("25-11-1951"));
		assertEquals("1951-11-25",DATE_FORMATTER.format(d));
		DATE_FORMATTER.parse(processor.fixDate("1957??-10-25"));
		DATE_FORMATTER.parse(processor.fixDate("2016?-sep-12"));
		DATE_FORMATTER.parse(processor.fixDate("2016-aou-26"));
		DATE_FORMATTER.parse(processor.fixDate("2016-avr-14"));
		DATE_FORMATTER.parse(processor.fixDate("2016-fev-03"));
		DATE_FORMATTER.parse(processor.fixDate("-","2016-01-01"));
	}

}
