from django.contrib.auth.models import User
from django.test import TestCase, RequestFactory
from django.urls import reverse

from web.views import *

from web.models import *

from web.tests.functions import *

import datetime


class NotificationViewTests(TestCase):
    def setUp(self):
        # Every test needs access to the request factory.
        self.factory = RequestFactory()
        self.user = User.objects.create_user(
            username='piotr', email='jacob@bla', password='top_secret')

    def test_get_exceeded_visit_notifications_count(self):
        original_notification = get_visits_without_appointments_count()

        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_end = "2011-01-01"
        visit.save()
        appointment = create_appointment(visit)

        notification = get_exceeded_visit_notifications_count()
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_exceeded_visit_notifications_count_2(self):
        original_notification = get_visits_without_appointments_count()

        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_end = "2011-01-01"
        visit.is_finished = True
        visit.save()
        appointment = create_appointment(visit)

        notification = get_exceeded_visit_notifications_count()
        self.assertEquals(original_notification.count, notification.count)

    def test_get_visits_without_appointments_count(self):
        original_notification = get_visits_without_appointments_count()
        subject = create_subject()
        visit = create_visit(subject)

        notification = get_visits_without_appointments_count()
        self.assertEquals(original_notification.count + 1, notification.count)


    def test_get_visits_without_appointments_count_3(self):
        original_notification = get_visits_without_appointments_count()
        subject = create_subject()
        visit = create_visit(subject)
        appointment = create_appointment(visit)

        appointment.status = Appointment.APPOINTMENT_STATUS_CANCELLED
        appointment.save()

        notification = get_visits_without_appointments_count()
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_approaching_visits_without_appointments_count(self):
        original_notification = get_approaching_visits_without_appointments_count()
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date()+datetime.timedelta(days=2)
        visit.save()

        notification = get_approaching_visits_without_appointments_count()
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_approaching_visits_without_appointments_count_2(self):
        original_notification = get_approaching_visits_without_appointments_count()
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date()+datetime.timedelta(days=2)
        visit.save()
        appointment = create_appointment(visit)

        notification = get_approaching_visits_without_appointments_count()
        self.assertEquals(original_notification.count, notification.count)

    def test_get_approaching_visits_without_appointments_count_3(self):
        original_notification = get_approaching_visits_without_appointments_count()
        subject = create_subject()
        visit = create_visit(subject)
        visit.datetime_begin = get_today_midnight_date()+datetime.timedelta(days=2)
        visit.save()
        appointment = create_appointment(visit)

        appointment.status = Appointment.APPOINTMENT_STATUS_CANCELLED
        appointment.save()

        notification = get_approaching_visits_without_appointments_count()
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_subject_with_no_visit_notifications_count(self):
        original_notification = get_subject_with_no_visit_notifications_count()
        subject = create_subject()

        notification = get_subject_with_no_visit_notifications_count()
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_subject_with_no_visit_notifications_count_2(self):
        original_notification = get_subject_with_no_visit_notifications_count()
        subject = create_subject()

        visit = create_visit(subject)
        visit.is_finished=True
        visit.save()

        notification = get_subject_with_no_visit_notifications_count()
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_subject_with_no_visit_notifications_count_3(self):
        original_notification = get_subject_with_no_visit_notifications_count()
        subject = create_subject()

        visit = create_visit(subject)

        notification = get_subject_with_no_visit_notifications_count()
        self.assertEquals(original_notification.count, notification.count)
    def test_get_subject_with_no_visit_notifications_count_4(self):
        original_notification = get_subject_with_no_visit_notifications_count()
        subject = create_subject()
        subject.dead = True
        subject.save()

        notification = get_subject_with_no_visit_notifications_count()
        self.assertEquals(original_notification.count, notification.count)
    def test_get_subject_with_no_visit_notifications_count_5(self):
        original_notification = get_subject_with_no_visit_notifications_count()
        subject = create_subject()
        subject.resigned = True
        subject.save()

        notification = get_subject_with_no_visit_notifications_count()
        self.assertEquals(original_notification.count, notification.count)

    def test_get_unfinished_appointments_count(self):
        original_notification = get_unfinished_appointments_count()
        subject = create_subject()
        visit = create_visit(subject)
        appointment = create_appointment(visit)
        appointment.datetime_when = "2011-01-01"
        appointment.status = Appointment.APPOINTMENT_STATUS_SCHEDULED
        appointment.save()

        notification = get_unfinished_appointments_count()
        self.assertEquals(original_notification.count + 1, notification.count)

    def test_get_unfinished_appointments_count_2(self):
        original_notification = get_unfinished_appointments_count()
        subject = create_subject()
        visit = create_visit(subject)
        appointment = create_appointment(visit)
        appointment.datetime_when = "2011-01-01"
        appointment.status = Appointment.APPOINTMENT_STATUS_CANCELLED
        appointment.save()

        notification = get_unfinished_appointments_count()
        self.assertEquals(original_notification.count, notification.count)
